#!/usr/bin/env python3
#  Copyright (c) 2020 Netflix.
#  All rights reserved.
#
import sys

import click
import click_completion.core
from ntscli_cloud_lib import click_sanitize_rae_names
from ntscli_cloud_lib.stateful_session import stateful_session_mgr

# Implementation libs
import ntsjson
from ntsjson import __version__
from ntsjson.batch_result_impl import batch_result_impl
from ntsjson.cancel_impl import cancel_impl
from ntsjson.completion_group import completion, completion_alpha, custom_startswith
from ntsjson.filter_impl import filter_impl
from ntsjson.functions import make_basic_options_dict, nonblock_target_write, set_log_levels_of_libs
from ntsjson.get_device_list_impl import get_device_list_impl
from ntsjson.get_plan_impl import get_plan_impl
from ntsjson.eyepatch_check_calibration_impl import eyepatch_check_calibration_impl
from ntsjson.eyepatch_calibrate_impl import eyepatch_calibrate_impl
from ntsjson.eleven_calibrate_impl import eleven_calibrate_impl
from ntsjson.log import logger, nts_verbosity_option
from ntsjson.run_impl import run_impl
from ntsjson.ssl_click_group import ssl_group
from ntsjson.status_impl import status_impl
from ntsjson.cli_required_if import RequiredIf
from ntsjson.cli_illegal_if import IllegalIf

mydir = ""
last_message = {}

RAE_HELP_STR = "The Netflix RAE device serial to connect to, such as r3010203. Defaults to environment variable 'RAE'."
ESN_HELP_STR = "The ESN of the target device."
IP_HELP_STR = "The IP address of the target device."
SERIAL_HELP_STR = "The serial number of the target device."
SSL_CONFIG_HELP_STR = "The security configuration directory to use from ~/.config/netflix. Defaults to 'cloud'."
SSL_ENV_VAR = "NTS_SSL_CONFIG_DIR"


@click.group(name="default")
@click.version_option(
    __version__.__version__,
    message="%(prog)s, build %(version)s",
    prog_name="NTS-CLI Installable Client 2.1.0"
)
def default():
    """
    NTS CLI 2.0 reference implementation

    Each command has its own --help option to show its usage.
    """
    set_log_levels_of_libs()


@default.command(name="has-eyepatch", short_help="Does an ESN have an EyePatch config?")
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--esn",
    type=str,
    help=ESN_HELP_STR,
)
@click.option(
    "--ip",
    type=str,
    help=IP_HELP_STR,
)
@click.option(
    "--serial",
    type=str,
    help=SERIAL_HELP_STR,
)
@click.option(
    "--save-to",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def is_esn_connected_to_eyepatch(rae, configuration, esn, ip, serial, save_to):
    """
    Test whether a device has an EyePatch configuration.
    """
    set_log_levels_of_libs()
    options = make_basic_options_dict(esn, ip, rae, serial, configuration)
    with stateful_session_mgr(**options) as session:
        # if you have the esn, don't bother
        if not esn:
            identifiers = session.get_device_list()
            if identifiers is None:
                logger.critical(
                    "We were unable to get the list of device identifiers to look up the ESN of your device. "
                    "This could be a connectivity issue, or the modules may not be installed on the RAE."
                )
                sys.exit(1)

            if ip:
                my_devices = [elt for elt in identifiers if elt.ip == ip]
                for elt in my_devices:
                    session.device.esn = elt.esn
            elif serial:
                my_devices = [elt for elt in identifiers if elt.serial == serial]
                for elt in my_devices:
                    session.device.esn = elt.esn

        # one last check
        if not session.device.esn:
            logger.critical(
                "We were unable to look up the ESN of your device based on what you gave us. "
                "Check that the value has not changed in the RAE web UI."
            )
            sys.exit(1)

        has_eyepatch = session.is_esn_connected_to_eyepatch()
        if has_eyepatch is None:
            logger.critical("The request failed to reach the Automator due to a bug."
                            " Definitely contact Netflix about this.")
        elif has_eyepatch:
            logger.info("This ESN has a configured EyePatch.")
            nonblock_target_write(save_to, '{"result":true}')
            sys.exit(0)
        else:
            logger.info(
                "This ESN is not in the list of devices with a configured EyePatch, but could simply be missing."
                " You can check with the get-devices command."
            )
            nonblock_target_write(save_to, '{"result":false}')
            sys.exit(1)


@default.command(name="get-devices", short_help="List devices behind a RAE")
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--save-to",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def get_device_list(*args, **kwargs):
    """
    List devices on a remote RAE.
    """
    get_device_list_impl(*args, **kwargs)


@default.command(name="run", short_help="Run a test plan")
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--esn",
    type=str,
    help=ESN_HELP_STR,
)
@click.option(
    "--ip",
    type=str,
    help=IP_HELP_STR,
)
@click.option(
    "--serial",
    type=str,
    help=SERIAL_HELP_STR,
)
@click.option("--batch", type=str, help="A batch name to locate this run in NTS")
@click.option(
    "--categories",
    type=str,
    help="An optional CSV field (remember quotes for spaces). Filter your test plan to these categories.",
)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--force-remove-eyepatch",
    is_flag=True,
    help="Remove EyePatch tests instead of querying the EyePatch module."
)
@click.option(
    "--force-keep-eyepatch",
    is_flag=True,
    help="Keep EyePatch tests, even if you do not have a configured EyePatch."
)
@click.option(
    "--names",
    type=str,
    help="Filter your tests to only names in a CSV list: 'AUDIO-001-TC1,DRS-AL1-25FPS-HEAAC-DWN'",
)
@click.option("--names-re", type=str, help="Filter your test names by regex: DRS.*")
@click.option(
    "--no-wait",
    is_flag=True,
    help="Return as soon as the tests are scheduled.",
)
@click.option("--print-during", is_flag=True, help="Print status updates while tests run.")
@click.option("--skip-print-after", is_flag=True, help="Skip printing the final status updates.")
@click.option(
    "--save-to",
    "result_file",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@click.option(
    "--tags",
    type=str,
    help="An optional CSV field. Filter your test plan to only include these tags.",
)
@click.option(
    "--max-nrdjs",
    is_flag=True,
    help="Set the max NRDJS flag on each test.",
)
@click.option(
    "--instrumentation-areas",
    type=str,
    help="An optional CSV field. Instrumentation areas to add to a test - dpi.DrmSystem,dpi.iCryptoAdapter",
)
@click.option(
    "--trace-areas",
    type=str,
    help="An optional CSV field. Trace areas to add to a test - CRYPTO,DRMSYSTEM",
)
@click.option(
    "--testplan",
    type=click.File("r", encoding="utf-8"),
    default=sys.stdin,
    help="JSON formatted test plan. Accepted from file or stdin.",
)
@click.option(
    "--retries",
    type=int,
    help="Number of times to retry failing tests.",
)
@click.option(
    "--branch",
    type=str,
    help="Use this alternate named branch hosted by Netflix.",
)
@click.option(
    "--pull-request",
    type=str,
    help="Use this alternate pull request number hosted by Netflix.",
)
@click.option(
    "--commit-hash",
    type=str,
    help="Use this alternate commit hash hosted by Netflix.",
)
@click.option(
    "--for-certification",
    is_flag=True,
    help="Use this for runs that you want counted against Netflix certification.",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def run(*args, **kwargs):
    """
    Run tests on a target device.

    \b
    Accepts test plans via --testplan or stdin.
    Results are written to stdout or a file specified by --save-to.
    Additional filtering can be done here, see the --help option.
    """
    run_impl(*args, **kwargs)


@click.group(name="alpha")
def alpha():
    """A alpha feature area. These are definitely not final."""


@alpha.command(name="filterplan", short_help="Filter a test plan")
@click.option(
    "--rae",
    type=str,
    help="Rewrite the destination RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--esn",
    type=str,
    help="Rewrite the destination ESN",
)
@click.option(
    "--ip",
    type=str,
    help="Rewrite the destination IP",
)
@click.option(
    "--serial",
    type=str,
    help="Rewrite the ADB serial number",
)
@click.option(
    "--testplan",
    type=click.File("r", encoding="utf-8"),
    default=sys.stdin,
    help="JSON formatted test plan. Accepted from file or stdin.",
)
@click.option("--batch", type=str, help="A batch name to locate this run in NTS")
@click.option(
    "--names",
    type=str,
    help="Filter your tests to only names in a CSV list: 'AUDIO-001-TC1,DRS-AL1-25FPS-HEAAC-DWN'",
)
@click.option("--names-re", type=str, help="Filter your test names by regex: DRS.*")
@click.option(
    "--categories",
    type=str,
    help="An optional CSV field (remember quotes for spaces). Filter your test plan to these categories.",
)
@click.option("--eyepatch", is_flag=True, help="Remove EyePatch tests, alias of --force-remove-eyepatch")
@click.option("--force-remove-eyepatch", is_flag=True, help="Remove EyePatch tests")
@click.option(
    "--save-to",
    "result_file",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@click.option(
    "--tags",
    type=str,
    help="An optional CSV field. Filter your test plan to only include these tags.",
)
@click.option(
    "--max-nrdjs",
    is_flag=True,
    help="Set the max NRDJS flag on each test.",
)
@click.option(
    "--instrumentation-areas",
    type=str,
    help="An optional CSV field. Instrumentation areas to add to a test - dpi.DrmSystem,dpi.iCryptoAdapter",
)
@click.option(
    "--trace-areas",
    type=str,
    help="An optional CSV field. Trace areas to add to a test - CRYPTO,DRMSYSTEM",
)
@click.option(
    "--branch",
    type=str,
    help="Use this alternate named branch hosted by Netflix.",
)
@click.option(
    "--pull-request",
    type=str,
    help="Use this alternate pull request number hosted by Netflix.",
)
@click.option(
    "--commit-hash",
    type=str,
    help="Use this alternate commit hash hosted by Netflix.",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def filterplan(*args, **kwargs):  # filter is a reserved word in python. careful!
    """
    Filter tests out of a test plan.

    \b
    Examples:

        cat plan.json | nts filter --names ACT-004-TC11,AUDIO-001-TC1 > filtered.json

        nts get-plan | nts filter --eyepatch > filtered.json

        nts filter --testplan plan.json --names ACT-004-TC11,AUDIO-001-TC1 | nts run
    """
    filter_impl(*args, **kwargs)


@default.command(name="cancel", short_help="Cancel the currently running test plan")
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--esn",
    type=str,
    help=ESN_HELP_STR,
)
@click.option(
    "--ip",
    type=str,
    help=IP_HELP_STR,
)
@click.option(
    "--serial",
    type=str,
    help=SERIAL_HELP_STR,
)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--save-to",
    "save_to",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@click.option(
    "--no-wait",
    is_flag=True,
    help="Return without waiting for device to become idle.",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def cancel(*args, **kwargs):
    """
    Cancel any pending tests for a specified device.

    By default this will also block this process until the device reports that it is idle, timing out after 30 retries.
    You can skip this wait by providing the --no-wait option.
    """
    cancel_impl(*args, **kwargs)


@default.command(name="get-plan", short_help="Get the test plan for a device")
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option("--esn", type=str, help=ESN_HELP_STR)
@click.option("--ip", type=str, help=IP_HELP_STR)
@click.option("--serial", type=str, help=SERIAL_HELP_STR)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--save-to",
    "testplan",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@click.option(
    "--playlist",
    type=str,
    help="UUID of a playlist from https://partnertools.nrd.netflix.com/nts/#playlist",
)
@click.option(
    "--type",
    "type_",  # shadows builtin "type" - underscore on purpose
    type=click.Choice(["dial", "adb"]),
    help="either dial or adb",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def get_plan_from_device(*args, **kwargs):
    """
    Have the target device retrieve its test plan and return it to the client.

    Run and filter commands will use a cached copy of this if no other input is given to them.
    Caches expire in one day and are stored in your user cache directory (~/.cache/ntscli-client or similar).
    """
    get_plan_impl(*args, **kwargs)


@default.command(
    name="eyepatch-check",
    short_help="Check eyepatch calibration."
)
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option("--esn", type=str, help=ESN_HELP_STR)
@click.option("--ip", type=str, help=IP_HELP_STR)
@click.option("--serial", type=str, help=SERIAL_HELP_STR)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--type",
    "type_",  # shadows builtin "type" - underscore on purpose
    type=click.Choice(["dial", "adb"]),
    help="either dial or adb",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def eyepatch_check(*args, **kwargs):
    """
    Have the target device retrieve the eyepatch check calibration test and run it.
    """
    eyepatch_check_calibration_impl(*args, **kwargs)


@default.command(
    name="eyepatch-calibrate",
    short_help="Calibrate the eyepatch for testing."
)
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option("--esn", type=str, help=ESN_HELP_STR)
@click.option("--ip", type=str, help=IP_HELP_STR)
@click.option("--serial", type=str, help=SERIAL_HELP_STR)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--frequency",
    cls=RequiredIf,
    required_if={'form-factor': 'set-top-box'},
    type=click.Choice(["50Hz", "59.94Hz", "60Hz"], case_sensitive=True),
    help="Many set top boxes will lock to specific video output frequency over HDMI.",
    envvar="EP_FREQUENCY",
)
@click.option(
    "--audio-source",
    required=True,
    type=click.Choice(["line-in", "contact-mic"], case_sensitive=True),
    help="The EyePatch sensor device supports a few audio capture input methods."
         " Line-in is preferred for audio quality verification and contact-mic is preferring for a/v sync testing.",
    envvar="EP_AUDIO_SOURCE",
)
@click.option(
    "--form-factor",
    required=True,
    type=click.Choice(["set-top-box", "smart tv"], case_sensitive=True),
    help="The EyePatch sensor device can be used to measure audio/video for either a Smart TV device or a "
         "Set Top Box device.",
    envvar="EP_FORM_FACTOR",
)
@click.option(
    "--topology",
    required=True,
    type=click.Choice(["STB->TV", "TV", "STB->AVR->TV", "TV->AVR"], case_sensitive=True),
    help="The a/v topology that is currently being configured for the device."
         " [\"STB->TV\" | \"TV\" | \"STB->AVR->TV\" | \"TV->AVR\"]."
         " [Value must be \"STB->TV\" when the --golden option is passed.]",
    envvar="EP_TOPOLOGY",
)
@click.option(
    "--audio-mode",
    required=True,
    type=click.Choice(["PASSTHROUGH", "ATMOS", "DDP51", "PCM", "DOLBYMAT"], case_sensitive=True),
    help="The audio mode.",
)
@click.option(
    "--eyepatch-serial",
    type=str,
    help="The serial number of the eyepatch to be used. [NOTE: required if multiple eyepatches are connected to the RAE]",
    envvar="EP_SERIAL",
)
@click.option(
    "--golden",
    cls=IllegalIf,
    illegal_if={'form-factor': 'smart tv'},
    illegal_if_not={'topology': 'STB->TV'},
    is_flag=True,
    help="Calibrate for eyepatch \"golden\" device setup.",
    envvar="EP_GOLDEN",
)
@click.option(
    "--dv-mode",
    cls=RequiredIf,
    required_if={'golden': True},
    type=click.Choice(["NA", "DV_STANDARD", "DV_LOWLATENCY"], case_sensitive=True),
    help="The Dolby Vision mode.",
)
@click.option(
    "--video-mode",
    cls=RequiredIf,
    required_if={'golden': True},
    type=click.Choice(["PASSTHROUGH", "ALWAYSHDR"], case_sensitive=True),
    help="The video mode.",
)
@click.option(
    "--hdr-mode",
    cls=RequiredIf,
    required_if={'golden': True},
    type=click.Choice(["HDR10", "DV_STANDARD", "DV_LOWLATENCY"], case_sensitive=True),
    help="The HDR mode.",
)
@click.option(
    "--reference-setup-version",
    cls=RequiredIf,
    required_if={'golden': True},
    type=str,
    help="The reference setup version.",
)
@click.option(
    "--type",
    "type_",  # shadows builtin "type" - underscore on purpose
    type=click.Choice(["dial", "adb"]),
    help="either dial or adb",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def eyepatch_calibrate(*args, **kwargs):
    """
    Have the target device retrieve the eyepatch calibration test and run it.
    """
    eyepatch_calibrate_impl(*args, **kwargs)


@default.command(
    name="eleven-calibrate",
    short_help="Calibrate the Eleven for testing."
)
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option("--esn", type=str, help=ESN_HELP_STR)
@click.option("--ip", type=str, help=IP_HELP_STR)
@click.option("--serial", type=str, help=SERIAL_HELP_STR)
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--type",
    "type_",  # shadows builtin "type" - underscore on purpose
    type=click.Choice(["dial", "adb"]),
    help="either dial or adb",
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def eleven_calibrate(*args, **kwargs):
    """
    Have the target device retrieve the Eleven calibration test and run it.

    """
    eleven_calibrate_impl(*args, **kwargs)


@default.command(
    name="status",
    short_help="Get the Automator in-memory status"
)  # the wrappers taint the name, set it back
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--esn",
    type=str,
    help=ESN_HELP_STR,
)
@click.option(
    "--ip",
    type=str,
    help=IP_HELP_STR,
)
@click.option(
    "--serial",
    type=str,
    help=SERIAL_HELP_STR,
)
@click.option(
    "--batch-id",
    "batch_id",
    type=str,
    help="The batch ID to get a result block for.",
)
@click.option(
    "--save-to",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def status(**kwargs):
    """
    Get the status of the Automator instance on a given RAE.

    This includes test sessions the Automator module has run since it was last restarted.
    The sessions include useful information about their state - running or not, which device they targeted, and more.
    """
    status_impl(**kwargs)


@default.command(
    name="batch-result",
    short_help="Get the results of a batch on a device"
)  # the wrappers taint the name, set it back
@click.option(
    "--config",
    "configuration",
    type=str,
    default="cloud",
    show_default=True,
    help=SSL_CONFIG_HELP_STR,
    envvar=SSL_ENV_VAR,
)
@click.option(
    "--rae",
    type=str,
    required=True,
    help=RAE_HELP_STR,
    envvar="RAE",
    callback=click_sanitize_rae_names,
)
@click.option(
    "--esn",
    type=str,
    help=ESN_HELP_STR,
)
@click.option(
    "--ip",
    type=str,
    help=IP_HELP_STR,
)
@click.option(
    "--serial",
    type=str,
    help=SERIAL_HELP_STR,
)
@click.option(
    "--save-to",
    type=click.File("w", encoding="utf-8"),
    help=ntsjson.SAVE_TO_HELP,
    default=sys.stdout,
)
@click.option("--skip-download", "-s", is_flag=True, help="Skip downloading and printing the full JSON report.")
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
@click.argument("batch_id", type=str, default="latest")
def batch_result(**kwargs):
    """
    Get the results of a batch on a device. Defaults to the latest found run for the given device.

    \b
    You may also provide a specific batch ID instead of "latest":
    nts batch-result ntscli-1FJLG2ZQ-2020-08-26T22:06:33.534Z --ip 192.168.144.70
    """
    batch_result_impl(**kwargs)


@default.command(name="print-update-command", short_help="Print the command to get updates to this client.")
@nts_verbosity_option(logger, default=ntsjson.VERBOSE_DEFAULT_LEVEL, help=ntsjson.VERBOSE_HELP)
def print_update_command():
    # logger.debug("debug!")
    # logger.info("info!")
    # logger.warning("warning!")
    # logger.error("error!")
    print('pip3 install -U "ntscli-client<3.0.0" --force')
    # logger.critical("crit!")


default.add_command(ssl_group)
# default.add_command(config_set_group)
default.add_command(alpha)
default.add_command(completion)
alpha.add_command(completion_alpha)

click_completion.core.startswith = custom_startswith
click_completion.init()

if __name__ == "__main__":  # pragma: no cover
    default()
